@@ -5,8 +5,6 @@ package processor
 import (
 	"encoding/base64"
 	"fmt"
-	jsoniter "github.com/json-iterator/go"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"runtime"
@@ -15,10 +13,14 @@ import (
 	"strconv"
 	"strings"
 	"sync"
+
+	"github.com/boyter/gocodewalker"
+
+	jsoniter "github.com/json-iterator/go"
 )
 
 // Version indicates the version of the application
-var Version = "3.0.0"
+var Version = "3.3.2"
 
 // Flags set via the CLI which control how the output is displayed
 
@@ -73,6 +75,9 @@ var More = false
 // Cocomo toggles the COCOMO calculation
 var Cocomo = false
 
+// Print a more SLOCCount like COCOMO calculation
+var SLOCCountFormat = false
+
 // CocomoProjectType allows the flipping between project types which impacts the calculation
 var CocomoProjectType = "organic"
 
@@ -91,9 +96,18 @@ var GitIgnore = false
 // Ignore disables ignore file checks
 var Ignore = false
 
+// CountIgnore should we count ignore files?
+var CountIgnore = false
+
 // DisableCheckBinary toggles checking for binary files using NUL bytes
 var DisableCheckBinary = false
 
+// UlocMode toggles checking for binary files using NUL bytes
+var UlocMode = false
+
+// Dryness toggles checking for binary files using NUL bytes
+var Dryness = false
+
 // SortBy sets which column output in formatter should be sorted by
 var SortBy = ""
 
@@ -118,6 +132,9 @@ var RemapUnknown = ""
 // RemapAll allows remapping of all files with a string to search the content for
 var RemapAll = ""
 
+// CurrencySymbol allows setting the currency symbol for cocomo project cost estimation
+var CurrencySymbol = ""
+
 // FileOutput sets the file that output should be written to
 var FileOutput = ""
 
@@ -139,9 +156,21 @@ var FileSummaryJobQueueSize = runtime.NumCPU()
 // AllowListExtensions is a list of extensions which are allowed to be processed
 var AllowListExtensions = []string{}
 
+// ExcludeListExtensions is a list of extensions which should be ignored
+var ExcludeListExtensions = []string{}
+
+// ExcludeFilename is a list of filenames which should be ignored
+var ExcludeFilename = []string{}
+
 // AverageWage is the average wage in dollars used for the COCOMO cost estimate
 var AverageWage int64 = 56286
 
+// Overhead is the overhead multiplier for corporate overhead (facilities, equipment, accounting, etc.)
+var Overhead float64 = 2.4
+
+// the effort adjustment factor derived from the cost drivers, i.e. 1.0 if rated nominal
+var EAF float64 = 1.0
+
 // GcFileCount is the number of files to process before turning the GC back on
 var GcFileCount = 10000
 var gcPercent = -1
@@ -267,6 +296,13 @@ func ProcessConstants() {
 			printTrace("configured to lazy load language features")
 		}
 	}
+
+	// Fix for https://github.com/boyter/scc/issues/250
+	fixedPath := []string{}
+	for _, path := range PathDenyList {
+		fixedPath = append(fixedPath, strings.TrimRight(path, "/"))
+	}
+	PathDenyList = fixedPath
 }
 
 // Configure and setup any count-as params the use has supplied
@@ -434,6 +470,10 @@ func processFlags() {
 		Generated = true
 	}
 
+	if Dryness {
+		UlocMode = true
+	}
+
 	if Debug {
 		printDebug(fmt.Sprintf("Path Deny List: %v", PathDenyList))
 		printDebug(fmt.Sprintf("Sort By: %s", SortBy))
@@ -448,6 +488,8 @@ func processFlags() {
 		printDebug(fmt.Sprintf("Minified/Generated Detection: %t/%t", Minified, Generated))
 		printDebug(fmt.Sprintf("Ignore Minified/Generated: %t/%t", IgnoreMinified, IgnoreGenerated))
 		printDebug(fmt.Sprintf("IncludeSymLinks: %t", IncludeSymLinks))
+		printDebug(fmt.Sprintf("Uloc: %t", UlocMode))
+		printDebug(fmt.Sprintf("Dryness: %t", Dryness))
 	}
 }
 
@@ -489,6 +531,11 @@ func printLanguages() {
 	}
 }
 
+// global variables to deal with ULOC calculations
+var ulocMutex = sync.Mutex{}
+var ulocGlobalCount = map[string]struct{}{}
+var ulocLanguageCount = map[string]map[string]struct{}{}
+
 // Process is the main entry point of the command line it sets everything up and starts running
 func Process() {
 	if Languages {
@@ -504,14 +551,24 @@ func Process() {
 		DirFilePaths = append(DirFilePaths, ".")
 	}
 
+	filePaths := []string{}
+	dirPaths := []string{}
+
 	// Check if the paths or files added exist and exit if not
 	for _, f := range DirFilePaths {
 		fpath := filepath.Clean(f)
 
-		if _, err := os.Stat(fpath); os.IsNotExist(err) {
+		s, err := os.Stat(fpath)
+		if os.IsNotExist(err) {
 			fmt.Println("file or directory does not exist: " + fpath)
 			os.Exit(1)
 		}
+
+		if s.IsDir() {
+			dirPaths = append(dirPaths, fpath)
+		} else {
+			filePaths = append(filePaths, fpath)
+		}
 	}
 
 	SortBy = strings.ToLower(SortBy)
@@ -522,30 +579,63 @@ func Process() {
 		printDebug(fmt.Sprintf("PathDenyList: %v", PathDenyList))
 	}
 
-	fileListQueue := make(chan *FileJob, FileListQueueSize)             // Files ready to be read from disk
-	fileSummaryJobQueue := make(chan *FileJob, FileSummaryJobQueueSize) // Files ready to be summarised
+	potentialFilesQueue := make(chan *gocodewalker.File, FileListQueueSize) // files that pass the .gitignore checks
+	fileListQueue := make(chan *FileJob, FileListQueueSize)                 // Files ready to be read from disk
+	fileSummaryJobQueue := make(chan *FileJob, FileSummaryJobQueueSize)     // Files ready to be summarised
+
+	fileWalker := gocodewalker.NewParallelFileWalker(dirPaths, potentialFilesQueue)
+	fileWalker.SetErrorHandler(func(e error) bool {
+		printError(e.Error())
+		return true
+	})
+	fileWalker.IgnoreGitIgnore = GitIgnore
+	fileWalker.IgnoreIgnoreFile = Ignore
+	fileWalker.IncludeHidden = true
+	fileWalker.ExcludeDirectory = PathDenyList
 
 	go func() {
-		directoryWalker := NewDirectoryWalker(fileListQueue)
+		err := fileWalker.Start()
+		if err != nil {
+			printError(err.Error())
+		}
+	}()
 
-		for _, f := range DirFilePaths {
-			err := directoryWalker.Start(f)
+	go func() {
+		for _, f := range filePaths {
+			fileInfo, err := os.Lstat(f)
 			if err != nil {
-				fmt.Printf("failed to walk %s: %v", f, err)
-				os.Exit(1)
+				continue
+			}
+
+			fileJob := newFileJob(f, f, fileInfo)
+			if fileJob != nil {
+				fileListQueue <- fileJob
 			}
 		}
 
-		directoryWalker.Run()
+		for fi := range potentialFilesQueue {
+			fileInfo, err := os.Lstat(fi.Location)
+			if err != nil {
+				continue
+			}
+
+			if !fileInfo.IsDir() {
+				fileJob := newFileJob(fi.Location, fi.Filename, fileInfo)
+				if fileJob != nil {
+					fileListQueue <- fileJob
+				}
+			}
+		}
+		close(fileListQueue)
 	}()
+
 	go fileProcessorWorker(fileListQueue, fileSummaryJobQueue)
 
 	result := fileSummarize(fileSummaryJobQueue)
-
 	if FileOutput == "" {
 		fmt.Println(result)
 	} else {
-		_ = ioutil.WriteFile(FileOutput, []byte(result), 0600)
+		_ = os.WriteFile(FileOutput, []byte(result), 0644)
 		fmt.Println("results written to " + FileOutput)
 	}
 }
